@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: filesystem.py 858 2008-03-17 02:46:45Z pronovic $
+# Revision : $Id: filesystem.py 879 2008-03-20 04:00:23Z pronovic $
 # Purpose  : Provides filesystem-related objects.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -52,6 +52,7 @@ import sys
 import os
 import re
 import sha
+import math
 import logging
 import tarfile
 
@@ -1305,8 +1306,9 @@ class PurgeItemList(FilesystemList):
       for entry in self[:]:
          if os.path.isfile(entry) and not os.path.islink(entry):
             try:
-               age = calculateFileAge(entry)
-               if age < daysOld:
+               ageInDays = calculateFileAge(entry)
+               ageInWholeDays = math.floor(ageInDays)
+               if ageInWholeDays < daysOld:
                   removed += 1
                   self.remove(entry)
             except OSError:
