@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2007 Kenneth J. Pronovici.
+# Copyright (c) 2004-2008 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # Portions copyright (c) 2001, 2002 Python Software Foundation.
@@ -30,7 +30,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: util.py 859 2008-03-18 02:49:50Z pronovic $
+# Revision : $Id: util.py 895 2008-03-20 20:25:20Z pronovic $
 # Purpose  : Provides general-purpose utilities.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -85,6 +85,8 @@ import time
 import logging
 import string
 
+from CedarBackup2.release import VERSION, DATE
+
 try:
    import pwd
    import grp
@@ -119,9 +121,9 @@ MBYTES_PER_GBYTE   = 1024.0
 BYTES_PER_MBYTE    = BYTES_PER_KBYTE * KBYTES_PER_MBYTE
 BYTES_PER_GBYTE    = BYTES_PER_MBYTE * MBYTES_PER_GBYTE
 
-SECONDS_PER_MINUTE = 60
-MINUTES_PER_HOUR   = 60
-HOURS_PER_DAY      = 24
+SECONDS_PER_MINUTE = 60.0
+MINUTES_PER_HOUR   = 60.0
+HOURS_PER_DAY      = 24.0
 SECONDS_PER_DAY    = SECONDS_PER_MINUTE * MINUTES_PER_HOUR * HOURS_PER_DAY
 
 UNIT_BYTES         = 0
@@ -941,6 +943,171 @@ else: # _PIPE_IMPLEMENTATION == "popen2.Popen4"
          self.tochild.close()       # we'll never write to it, and this way we don't confuse anything.
 
 
+########################################################################
+# Diagnostics class definition
+########################################################################
+
+class Diagnostics(object):
+
+   """
+   Class holding runtime diagnostic information.
+
+   Diagnostic information is information that is useful to get from users for
+   debugging purposes.  I'm consolidating it all here into one object.
+
+   @sort: __init__, __repr__, __str__, __cmp__
+   """
+
+   def __init__(self):
+      """
+      Constructor for the C{Diagnostics} class.
+      """
+
+   def __repr__(self):
+      """
+      Official string representation for class instance.
+      """
+      return "Diagnostics()"
+
+   def __str__(self):
+      """
+      Informal string representation for class instance.
+      """
+      return self.__repr__()
+
+   def getValues(self):
+      """
+      Get a map containing all of the diagnostic values.
+      @return: Map from diagnostic name to diagnostic value.
+      """
+      values = {}
+      values['version'] = self.version
+      values['interpreter'] = self.interpreter
+      values['platform'] = self.platform
+      values['encoding'] = self.encoding
+      values['locale'] = self.locale
+      values['timestamp'] = self.timestamp
+      return values;
+
+   def printDiagnostics(self, fd=sys.stdout, prefix=""):
+      """
+      Pretty-print diagnostic information to a file descriptor.
+      @param fd: File descriptor used to print information.
+      @param prefix: Prefix string (if any) to place onto printed lines
+      @note: The C{fd} is used rather than C{print} to facilitate unit testing.
+      """
+      lines = self._buildDiagnosticLines(prefix)
+      for line in lines:
+         fd.write("%s\n" % line)
+
+   def logDiagnostics(self, method, prefix=""):
+      """
+      Pretty-print diagnostic information using a logger method.
+      @param method: Logger method to use for logging (i.e. logger.info)
+      @param prefix: Prefix string (if any) to place onto printed lines
+      """
+      lines = self._buildDiagnosticLines(prefix)
+      for line in lines:
+         method("%s" % line)
+
+   def _buildDiagnosticLines(self, prefix=""):
+      """
+      Build a set of pretty-printed diagnostic lines.
+      @param prefix: Prefix string (if any) to place onto printed lines
+      @return: List of strings, not terminated by newlines.
+      """
+      values = self.getValues()
+      keys = values.keys()
+      keys.sort()
+      tmax = Diagnostics._getMaxLength(keys) + 3  # three extra dots in output
+      lines = []
+      for key in keys:
+         title = key.title()
+         title += (tmax - len(title)) * '.'
+         value = values[key]
+         line = "%s%s: %s" % (prefix, title, value)
+         lines.append(line)
+      return lines
+
+   def _getMaxLength(values):
+      """
+      Get the maximum length from among a list of strings.
+      """
+      tmax = 0
+      for value in values:
+         if len(value) > tmax:
+            tmax = len(value)
+      return tmax
+   _getMaxLength = staticmethod(_getMaxLength)
+
+   def _getVersion(self):
+      """
+      Property target to get the Cedar Backup version.
+      """
+      return "Cedar Backup %s (%s)" % (VERSION, DATE)
+
+   def _getInterpreter(self):
+      """
+      Property target to get the Python interpreter version.
+      """
+      version = sys.version_info
+      return "Python %d.%d.%d (%s)" % (version[0], version[1], version[2], version[3])
+
+   def _getEncoding(self):
+      """
+      Property target to get the filesystem encoding.
+      """
+      return sys.getfilesystemencoding() or sys.getdefaultencoding()
+
+   def _getPlatform(self):
+      """
+      Property target to get the operating system platform.
+      """
+      try:
+         if sys.platform.startswith("win"):
+            WINDOWS_PLATFORMS = [ "Windows 3.1", "Windows 95/98/ME", "Windows NT/2000/XP", "Windows CE", ]
+            wininfo = sys.getwindowsversion()
+            winversion = "%d.%d.%d" % (wininfo[0], wininfo[1], wininfo[2])
+            winplatform = WINDOWS_PLATFORMS[wininfo[3]]
+            wintext = wininfo[4]  # i.e. "Service Pack 2"
+            return "%s (%s %s %s)" % (sys.platform, winplatform, winversion, wintext)
+         else:
+            uname = os.uname()
+            sysname = uname[0] # i.e. Linux
+            release = uname[2] # i.e. 2.16.18-2
+            machine = uname[4] # i.e. i686
+            return "%s (%s %s %s)" % (sys.platform, sysname, release, machine)
+      except:
+         return sys.platform
+
+   def _getLocale(self):
+      """
+      Property target to get the default locale that is in effect.
+      """
+      try:
+         import locale
+         return locale.getdefaultlocale()[0]
+      except:
+         return "(unknown)"
+   
+   def _getTimestamp(self):
+      """
+      Property target to get a current date/time stamp.
+      """
+      try:
+         import datetime
+         return datetime.datetime.utcnow().ctime() + " UTC"
+      except:
+         return "(unknown)"
+
+   version = property(_getVersion, None, None, "Cedar Backup version.")
+   interpreter = property(_getInterpreter, None, None, "Python interpreter version.")
+   platform = property(_getPlatform, None, None, "Platform identifying information.")
+   encoding = property(_getEncoding, None, None, "Filesystem encoding that is in effect.")
+   locale = property(_getLocale, None, None, "Locale that is in effect.")
+   timestamp = property(_getTimestamp, None, None, "Current timestamp.")
+
+
 ########################################################################
 # General utility functions
 ########################################################################
@@ -1415,13 +1582,14 @@ def calculateFileAge(file):
 
    @param file: Path to a file on disk.
 
-   @return: Age of the file in days.
+   @return: Age of the file in days (possibly fractional).
    @raise OSError: If the file doesn't exist.
    """
    currentTime = int(time.time())
    fileStats = os.stat(file)
    lastUse = max(fileStats.st_atime, fileStats.st_mtime)  # "most recent" is "largest" 
-   ageInDays = (currentTime - lastUse) / SECONDS_PER_DAY
+   ageInSeconds = currentTime - lastUse
+   ageInDays = ageInSeconds / SECONDS_PER_DAY
    return ageInDays
 
 
